
%%%%%%%%%%%%%%%%%%%%%%%%%%% DLT_CALIBRATION_16p %%%%%%%%%%%%%%%%%%%%%%%%%


% Grgoire Linart, 2015, ULg
% Dpartement de gomatique et gomtrologie



% Ce script utilise la thorie de la DLT pour calibrer une prises de vue.

% Les inputs sont
%    - la taille des images  restituer
%    - Les coordonnes mondes de Points de Calibration (PC) (.xlsx)
%    - Les coordones-images de ces PC (.xlsx)
%
% A l'issue de ce script sont obtenus
%    - Les 16 paramtres de modlisation de la prise de vue
%    - La position du point principal de l'image
%    - La qualit de la modlisation (RMSE)
%
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc
clear all
format short e
tic

%% Importation des donnes


%Entrer la taille de l'image (en pixels)
limax=3264;
colmax=4912;

%Importation du .xlsx des coordonnes_monde.
com=xlsread('lego_PC_monde.xlsx');

% Importation du .xlsx des coordonnes_image.
coi=xlsread('legoL_PC.xlsx');


indexcoi=find(coi(:,1)~=-1);         %ligne pour lesquelles il y a une valeur.

%supprimer les lignes de coordonnes monde pour lesquels il y a un -1 dans
%la matrices coordonnes image.


index=find(coi(:,1)==-1);   %Trouver o sont les -1 de coi. A quelle ligne de coi?

sizeindex=length(index);



for z=sizeindex:-1:1;
    com(index(z,1),:)=[];   %Supprimer les lignes de com qui correspondent aux valeurs de l'index.
    coi(index(z,1),:)=[];   %Supprimer les lignes de coi qui correspondent aux valeurs de l'index.
end


%% Ajouts d'errerus alatoires

% Erreurs de digitalisation

%for l=1:sizeindex
%   a=-1;
%   b=1;
%    coi(l,1)=coi(l,1)+ round(a + (b-a)*rand);
%    coi(l,2)=coi(l,2)+ round(a + (b-a)*rand);
%end

% Erreurs coordonnes-mondes

%for l=1:sizeindex
%   a=-0.002;
%   b=0.002;
%   c=-0.003;
%   d=0.003;
%    com(l,1)=com(l,1)+ round(a + (b-a)*rand);
%    com(l,2)=com(l,2)+ round(a + (b-a)*rand);
%    com(l,3)=com(l,3)+ round(c + (d-c)*rand);
%end


sizept=length(com);




%% 2) Dtermination des 16 paramtres


ite=50  ;                      % Entrer le nombre d'itrations demand 


%Prallocation de la mmoire.
Uo=zeros(1,1,ite+1);
Vo=zeros(1,1,ite+1);
Vo(1,1,1)=limax/2;
Uo(1,1,1)=colmax/2;

xi=zeros(sizept,1,ite);
eta=zeros(sizept,1,ite);
rsquare=zeros(sizept,1,ite);



% Nouvelle allocation des L, B, M et R
L=zeros(16,1,ite);
B=zeros(2*sizept,1);
M=zeros(2*sizept,16,ite);
R=ones(sizept,1,ite);         %R  1

%Allocation de la mmoire pour la QR decompostion
Qb=zeros(2*sizept,2*sizept,ite);
triRb=zeros(2*sizept,16,ite);
Q=zeros(2*sizept,16,ite);
triR=zeros(16,16,ite);

temp=zeros(16,1,ite);


%Allocation de la mmoire pour les erreurs de distorsion (delta u et delta v)
ErrorDistoU=zeros(sizept,1,ite);
ErrorDistoV=zeros(sizept,1,ite);

% Allocation de la mmoire pour les erreurs de calibration (kwon)
ErrorU=zeros(sizept,1,ite);
ErrorV=zeros(sizept,1,ite);

ErrorCalibration=zeros(sizept,1,ite);
ErrorCalibrationGlobal=zeros(1,1,ite);

ErrorUsansDisto=zeros(sizept,1,ite);
ErrorVsansDisto=zeros(sizept,1,ite);

ErrorCalibrationsansDisto=zeros(sizept,1,ite);
ErrorCalibrationGlobalsansDisto=zeros(1,1,ite);


%Allocation de la mmoire pour la position de la camera

m1=zeros(3,3,ite);
m2=zeros(3,1,ite);
posicamxyz=zeros(3,1,ite);

%% 

%Calcul des epsi et psi (ce qui revient  un changement d'axe de rf)

for n=1:ite
    
    xi(:,1,n)=coi(:,1,1)-Uo(:,1,n);
    eta(:,1,n)=coi(:,2,1)-Vo(:,1,n);
    rsquare(:,1,n)=xi(:,1,n).^2+ eta(:,1,n).^2  ;
                                   
    
% construction de la matrice B

B=reshape(flipud(rot90(coi)),numel(coi),1);

% construction de la matrice M

        for i=1:sizept
          M(2*i-1,1:3,n)=com(i,1:3);
          M(2*i,5:7,n)=com(i,1:3);
          M(2*i-1,4,n)=1;
          M(2*i,8,n)=1;
          M(2*i-1,9:11,n)=(com(i,1:3).*-coi(i,1));
          M(2*i,9:11,n)=(com(i,1:3).*-coi(i,2));
          
          M(2*i-1,12,n)=xi(i,:,n)*rsquare(i,:,n)*R(i,1,n);
          M(2*i,12,n)=eta(i,:,n)*rsquare(i,:,n)*R(i,1,n);
          
          M(2*i-1,13,n)=xi(i,:,n)*rsquare(i,:,n)^2*R(i,1,n);
          M(2*i,13,n)=eta(i,:,n)*rsquare(i,:,n)^2*R(i,1,n);
          
          M(2*i-1,14,n)=xi(i,:,n)*rsquare(i,:,n)^3*R(i,1,n);
          M(2*i,14,n)=eta(i,:,n)*rsquare(i,:,n)^3*R(i,1,n);
          
          M(2*i-1,15,n)=(rsquare(i,:,n)+2*xi(i,:,n)^2)*R(i,1,n);
          M(2*i,15,n)=xi(i,:,n)*eta(i,:,n)*R(i,1,n);
          
          M(2*i-1,16,n)=xi(i,:,n)*eta(i,:,n)*R(i,1,n);
          M(2*i,16,n)=(rsquare(i,:,n)+2*eta(i,:,n)^2)*R(i,1,n);
          
        end



% Rsolution du sytme par moindres carrs.




[Qb(:,:,n),triRb(:,:,n)]=qr(M(:,:,n));     % compute full QR factorization

Q(:,:,n) = Qb(:,1:16,n);                   
triR(:,:,n) = triRb(1:16,:,n);             

temp(:,:,n)=transpose(Q(:,:,n))*B;
 
L(:,1,n)=backsub(triR(:,:,n),temp(:,:,n));             %back substitution


%Dtermination de la position de la camera.

%m1(:,:,n)=inv([L(1,1,n),L(2,1,n),L(3,1,n);L(5,1,n),L(6,1,n),L(7,1,n); ...
  %L(9,1,n),L(10,1,n),L(11,1,n)]);
%m2(:,:,n)=[-L(4,1,n);-L(8,1,n);-1];

%posicamxyz(:,:,n)=(m1(:,:,n))*m2(:,:,n);



%Calcul du nouveaux R, pour l'itration suivante.
R(:,1,n+1)=L(9,1,n)*com(:,1)+L(10,1,n)*com(:,2)+L(11,1,n)*com(:,3)+1;


%Points principaux
Dsq=1/((L(9,1,n))^2 + (L(10,1,n))^2  + (L(11,1,n))^2  );
Uo(1,1,n+1)=Dsq*(L(1,1,n)*L(9,1,n)+L(2,1,n)*L(10,1,n)+L(3,1,n)*L(11,1,n));
Vo(1,1,n+1)=Dsq*(L(5,1,n)*L(9,1,n)+L(6,1,n)*L(10,1,n)+L(7,1,n)*L(11,1,n));



%% Erreurs optiques (en pixels)

% Rien d'utilisable car on ne sait pas o sont les points de
% calibration... On s'en sert pour la suite.
% On peut observer les distorsions max en u et v (en pixels).

%Computation
        for i=1:sizept
            ErrorDistoU(i,1,n)=xi(i,1,n)*(L(12,1,n)*rsquare(i,1,n)+L(13,1,n)*rsquare(i,1,n).^2+L(14,1,n)*rsquare(i,1,n).^3)+...
                L(15,1,n)*(rsquare(i,:,n)+2*xi(i,:,n).^2)+L(16,1,n)*xi(i,:,n)*eta(i,:,n);

            ErrorDistoV(i,1,n)=eta(i,1,n)*(L(12,1,n)*rsquare(i,1,n)+L(13,1,n)*rsquare(i,1,n).^2+L(14,1,n)*rsquare(i,1,n).^3)+...
                L(15,1,n)*xi(i,:,n)*eta(i,:,n)+L(16,1,n)*(rsquare(i,:,n)+2*eta(i,:,n).^2);

        end
        



%% Rsidus de calibration

for i=1:sizept
    
    %Avec prise en compte de la distorsion

ErrorU(i,1,n)=coi(i,1,1)-ErrorDistoU(i,1,n)-((L(1,1,n)*com(i,1,1)+L(2,1,n)*com(i,2,1)+L(3,1,n)*com(i,3,1)+L(4,1,n))/...
    (L(9,1,n)*com(i,1,1)+L(10,1,n)*com(i,2,1)+L(11,1,n)*com(i,3,1)+1));

ErrorV(i,1,n)=coi(i,2,1)-ErrorDistoV(i,1,n)-((L(5,1,n)*com(i,1,1)+L(6,1,n)*com(i,2,1)+L(7,1,n)*com(i,3,1)+L(8,1,n))/...
    (L(9,1,n)*com(i,1,1)+L(10,1,n)*com(i,2,1)+L(11,1,n)*com(i,3,1)+1));


ErrorCalibration(i,1,n)=(ErrorU(i,1,n).^2+ErrorV(i,1,n).^2)^0.5;


    %Sans prendre en compte la distorsion.

ErrorUsansDisto(i,1,n)=coi(i,1,1)-((L(1,1,n)*com(i,1,1)+L(2,1,n)*com(i,2,1)+L(3,1,n)*com(i,3,1)+L(4,1,n))/...
    (L(9,1,n)*com(i,1,1)+L(10,1,n)*com(i,2,1)+L(11,1,n)*com(i,3,1)+1));

ErrorVsansDisto(i,1,n)=coi(i,2,1)-((L(5,1,n)*com(i,1,1)+L(6,1,n)*com(i,2,1)+L(7,1,n)*com(i,3,1)+L(8,1,n))/...
    (L(9,1,n)*com(i,1,1)+L(10,1,n)*com(i,2,1)+L(11,1,n)*com(i,3,1)+1));


ErrorCalibrationsansDisto(i,1,n)=(ErrorUsansDisto(i,1,n).^2+ErrorVsansDisto(i,1,n).^2)^0.5;


end

ErrorCalibrationGlobal(1,1,n)=sum(ErrorCalibration(:,1,n))/sizept;
ErrorCalibrationGlobalb=reshape(ErrorCalibrationGlobal(1,1,:),ite,1);   % pour rorganiser en une seule matrice les rsultats


ErrorCalibrationGlobalsansDisto(1,1,n)=sum(ErrorCalibrationsansDisto(:,1,n))/sizept;
ErrorCalibrationGlobalbsansDisto=reshape(ErrorCalibrationGlobalsansDisto(1,1,:),ite,1); 
    
end
    
 
%% Plot de l'volution des paramtres au cours des itrations.
% ou plutot de l'cart entre la somme de ceux-ci pour chaque itration.

sumL=zeros(ite,1);
diffL=zeros(ite-1,1);
for i=1:ite;
sumL(i,1)=sum(L(:,1,i));

end

for i=1:ite-1;
diffL(i,1)=sumL(i,1)-sumL(i+1,1);
end
diffLcol=(diffL);
diffL=sqrt(abs(diffL))';

convLcolo=sqrt(abs(diffL))';
convL=plot(1:ite-1,diffL)   ;  %sqrt pour rduire les valeurs la valeur de l''iteration1
title('Evolution de l''indice de convergence');
xlabel('Nombre d''itration');
ylabel('Indice'); 
    



 %% Plot de l'cart pour chacun des points de calibration entre u,v mesur et u,v calcul avec prise en compte disto.
 %Seulement pour la dernire itration.
 
 
figure; 

bbar=bar(indexcoi,ErrorCalibration(:,1,ite));
Residus=ErrorCalibration(:,1,ite);

get(bbar);
set(bbar,'faceColor',[0 0.5 0.5],...
    'EdgeColor','w',...
    'DisplayName', 'Figure - Rsidus');


xlabel('Point de calibration');
ylabel('Rsidus (pixels)'); 
title('Ecart en (pixel) par point de calibration avec prise en compte la disto')



 %% Plot de l'cart pour chacun des points de calibration entre u,v mesur et u,v calcul sans disto disto.
 %Seulement pour la dernire itration.
 
 
figure; 

bbar=bar(indexcoi,ErrorCalibrationsansDisto(:,1,ite));
ResidusSansDisto=ErrorCalibrationsansDisto(:,1,ite);

get(bbar);
set(bbar,'faceColor',[0 0.5 0.5],...
    'EdgeColor','w',...
    'DisplayName', 'Figure - Rsidus');


xlabel('Point de calibration');
ylabel('Rsidus (pixels)'); 
title('Ecart en (pixel) par point de calibration sans prendre en compte la distorsion');




%% Dtermination des grandeurs physique

[posiCam,T,ypr,PPuo,PPvo,Z]=DLTcameraPosition(L(:,:,ite));
PP=[PPuo PPvo];


%% Exportation des donnes essentielles pour la restitution.


%les 16 paramtres

filenameL='legoL_L.xlsx';
AL=L(:,1,ite);
exportL=xlswrite(filenameL,AL);

%les coordonnes du points principal

filenamePP='legoL_PP.xlsx';
exportPP=xlswrite(filenamePP,PP);


%% autres exportations
    
%Exportation des erruers de distorsion.
%DistoU=ErrorDistoU(:,1,ite);
%DistoV=ErrorDistoV(:,1,ite);

%filename1='DistorsionOptiqueUlegoL_PC';
%exporterreurdistoU=xlswrite(filename1,ErrorDistoU(:,:,ite));

%filename2='DistorsionOptiqueVlegoL_PC';
%exporterreurdistoV=xlswrite(filename2,ErrorDistoV(:,:,ite));

%Exportation psi epsi
%filename3='epsi_(u-uo)legoL_PC';
%xiExport=xi(:,:,ite)
%exportepsi=xlswrite(filename3,xiExport);

%filename4='psi_(v-vo)legoL_PC';
%etaExport=eta(:,:,ite)
%exportpsi=xlswrite(filename4,etaExport);

%Exportation rapport erreurs
%Residus=ErrorCalibration(:,1,ite);
%ResidusU=ErrorU(:,1,ite);
%ResidusV=ErrorV(:,1,ite);



%exporterrorCalibrationGlobale=xlswrite('Error_calib_Kwon_legoL',ErrorCalibrationGlobalb);

%exportResidus=xlswrite('Residus_legoC_PC',ErrorCalibration(:,1,ite));

%rsidus en u et en v (avec modlisation de la distorsion)
residuU=ErrorU(:,:,ite);
residuV=ErrorV(:,:,ite);

%Rsidus en u et en v (avec modolisation de la distorsion)
residuSanDistoU=ErrorUsansDisto(:,:,ite);
residuSansDistoV=ErrorVsansDisto(:,:,ite);
toc

disp('Calibration effectue')