
%%%%%%%%%%%%%%%%%%%%%%%%%%% DLT_RESTITUTION_16p %%%%%%%%%%%%%%%%%%%%%%%%%


% Grgoire Linart, 2015, ULg
% Dpartement de gomatique et gomtrologie



% Ce script utilise la thorie de la DLT pour restituer des points en 3D.

% Les inputs sont
%    - Le nombre de camras utilises
%    - Le nombre de points homologues  restituer
%    - Les 16 paramtres pour chaque prise de vue issues du script
%    DLT_CALIBRATION_16p (.csv)
%    - Le Point Principal de chaque prise de vue issu du script
%    DLT_CALIBRATION_16p (.csv)
%    - Les coordones-images des points  restituer (ou des PV) (.xlsx)

% A l'issue de l'xcution du script, la position tridimensionnelle des 
% points  restiuer est obtenue (XYZ.csv) ainsi qu'un rapport d'erreurs.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all
clc
format short e



%% Importation des donnes

nCam=4       %nombre de camras
nHo=12       %nombe de points homologues maximum (PV pour le moment)



%Coordonnes monde des points de vrification.
GCPmonde=xlsread('exp1_PV_monde.xlsx');

%Paramtres L
L=zeros(16,1,nCam);

L(:,1,1)=csvread('exp1_50d_L.csv');
L(:,1,2)=csvread('exp1_400d_L.csv');
L(:,1,3)=csvread('exp1_kodak_L.csv');
L(:,1,4)=csvread('exp1_nex_L.csv');
%etc selon le nomnre de camras

%Coordonnes du points principal pour chauqe image
PP=zeros(1,2,nCam);

PP(:,:,1)=csvread('exp1_50d_PP.csv');
PP(:,:,2)=csvread('exp1_400d_PP.csv');
PP(:,:,3)=csvread('exp1_kodak_PP.csv');
PP(:,:,4)=csvread('exp1_nex_PP.csv');
%etc selon le nomnre de camras


%Points homologues (ici les points de vrification pour le moment)
hom=zeros(nHo,2,nCam);

hom(:,:,1)=xlsread('exp1_PV_50d.xlsx');               
hom(:,:,2)=xlsread('exp1_PV_400d.xlsx');
hom(:,:,3)=xlsread('exp1_PV_kodak.xlsx');
hom(:,:,4)=xlsread('exp1_PV_nex.xlsx');
%etc selon le nomnre de camras

%Trouver o se trouvent les -1 (pas de valeur)

index=find(hom(:,:,:)==-1);   


%% Ajout d'erreurs alatoires de digitalisation (fentre 25 pixels)


%for l=1:nHo
%    for k=1:nCam
%   a=-2;
%   b=2;
%    hom(l,1,k)=hom(l,1,k)+ round(a + (b-a)*rand);
%    hom(l,2,k)=hom(l,2,k)+ round(a + (b-a)*rand);
%    end
%end


%% Restitution

%Allocation de la mmoire
xi=zeros(nHo,1,nCam);
eta=zeros(nHo,1,nCam);
rsquare=zeros(nHo,1,nCam);

ErrorDistoU=zeros(nHo,1,nCam);
ErrorDistoV=zeros(nHo,1,nCam);

muom=zeros(nHo,2,nCam);
muomSansDisto=zeros(nHo,2,nCam);

xyz=zeros(1:nHo,1:3);

%% Prparation des variables

%calcul des xi et eta et rsquare

for n=1:nCam

    
xi(:,1,n)=hom(:,1,n)-PP(1,1,n);
eta(:,1,n)=hom(:,2,n)-PP(1,2,n);
rsquare(:,1,n)=xi(:,1,n).^2+ eta(:,1,n).^2;



%Calcul des distorsions radiales.

        for i=1:nHo

            ErrorDistoU(i,1,n)=xi(i,1,n)*(L(12,1,n)*rsquare(i,1,n)+L(13,1,n)*rsquare(i,1,n).^2+L(14,1,n)*rsquare(i,1,n).^3)+...
                L(15,1,n)*(rsquare(i,:,n)+2*xi(i,:,n).^2)+L(16,1,n)*xi(i,:,n)*eta(i,:,n);

            ErrorDistoV(i,1,n)=eta(i,1,n)*(L(12,1,n)*rsquare(i,1,n)+L(13,1,n)*rsquare(i,1,n).^2+L(14,1,n)*rsquare(i,1,n).^3)+...
                L(15,1,n)*xi(i,:,n)*eta(i,:,n)+L(16,1,n)*(rsquare(i,:,n)+2*eta(i,:,n).^2);       

        end
        
        
muom(:,1,n)= hom(:,1,n)-ErrorDistoU(:,1,n);          
muom(:,2,n)=hom(:,2,n)-ErrorDistoV(:,1,n);

muomSansDisto(:,1,n)=hom(:,1,n)
muomSansDisto(:,2,n)=hom(:,2,n)

end

sizeindex=length(index);




for z=sizeindex:-1:1;
    muom(index(z,1))=NaN;   %Remettre le NaN quand il n'y a pas de valeur.
end

%Rorganisation de la matrice xi eta (muom), pour des facilits de calcul.

muomb=reshape(muom,nHo,2*nCam);          %changer si on veut ou non pendre la disto en compte nuom ou muom sans disto.

for i=1:nHo
  
  % Liste des camras pour lesquels il n' a pas de NaN.
  camlist=find(isnan(muomb(i,1:2:nCam*2))==false);
  
  if numel(camlist)<2
      
    
    disp('plus de deux camras est ncssaire')
    
    
  % Si il y a plus de deux camras, on peut reconstruire.
  elseif numel(camlist)>=2
 
    m1=[];
    m2=[];

    m1(1:2:numel(camlist)*2,1)=muomb(i,camlist*2-1).*L(9,camlist)-L(1,camlist);
    m1(1:2:numel(camlist)*2,2)=muomb(i,camlist*2-1).*L(10,camlist)-L(2,camlist);
    m1(1:2:numel(camlist)*2,3)=muomb(i,camlist*2-1).*L(11,camlist)-L(3,camlist);
    
    m1(2:2:numel(camlist)*2,1)=muomb(i,camlist*2).*L(9,camlist)-L(5,camlist);
    m1(2:2:numel(camlist)*2,2)=muomb(i,camlist*2).*L(10,camlist)-L(6,camlist);
    m1(2:2:numel(camlist)*2,3)=muomb(i,camlist*2).*L(11,camlist)-L(7,camlist);
    

    m2(1:2:numel(camlist)*2,1)=L(4,camlist)-muomb(i,camlist*2-1);
    m2(2:2:numel(camlist)*2,1)=L(8,camlist)-muomb(i,camlist*2);

    
    xyz(i,1:3)=mldivide(m1,m2);
  
  end
  
  
end


%% controle qualit


ErrorX=xyz(:,1)-GCPmonde(:,1);                                %rsidus en x
ErrorY=xyz(:,2)-GCPmonde(:,2);                                %rsidus en y
ErrorZ=xyz(:,3)-GCPmonde(:,3);                                %rsidus en z
ErrorGlobal=(sum((ErrorX.^2+ErrorY.^2+ErrorZ.^2).^0.5))/nHo;  %RMSE

ErrorXYZ=(ErrorX.^2+ErrorY.^2+ErrorZ.^2).^0.5;  %distance d'erreur pour PV


%Plot

figure; 

bbar16=bar(ErrorXYZ);
hold off
get(bbar16);
set(bbar16,'faceColor',[0 0.5 0.5],...
    'EdgeColor','w',...
    'DisplayName', 'Figure - Rsidus GCP L16');


xlabel('Points de vrfication');
ylabel('Rsidus L16 (m)'); 
title('Rsidus par GCP - L16 avec prise en compte la distorsion!!!');


hold off


%% Exportations

%Exportation des points restitus

  exportXYZ=xlswrite('XYZ',xyz);  
  
%Exportation des rapports d'erreurs
  exportErrorX=xlswrite('residusX',ErrorX);
  exportErrorY=xlswrite('residusY',ErrorY);
  exportErrorZ=xlswrite('residusZ',ErrorZ);
  
  exportErrorXYZ=xlswrite('ErrorXYZ',ErrorXYZ);
  
  
  exportErrorGlobal=xlswrite('RMSE',ErrorGlobal);  
  
  disp('Restitution effectue')
